/*
The software is provided by the Institute of Commercial Cryptography Standards 
(ICCS), and is used for algorithm submissions in the Next-generation Commercial 
Cryptographic Algorithms Program. 

ICCS doesn't represent or warrant that the operation of the software will be 
uninterrupted or error-free in all cases. ICCS will take no responsibility for 
the use of the software or the results thereof, if the software is used for any 
other purposes.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "drng.h"
#include "KEX_AlgorithmInstance.h"

#define SEED_LENGTH_BYTE 64

#define KAT_KEX_SUCCESS 0
#define KAT_KEX_FOPEN_ERROR -1
#define KAT_KEX_CRYPTO_FAILURE -3

void fprintstr(FILE *file_output, unsigned char *msg, unsigned long long len);

// DRNG_ctx for generating pseudorandom numbers within the KEX protocol
DRNG_ctx drng_algorithm;

/****************output KAT_KEX_AlgorithmInstance.txt****************/
int main()
{
	unsigned char *nonce;
	// DRNG_ctx for generating seed
	DRNG_ctx drng_seed;
	char algoname_output[128];
	FILE *file_output;
	unsigned char *seed, *pka, *ska, *pkb, *skb, *pa, *pb, *ssa, *ssb, *ma, *mb;
	unsigned long long round, pka_byts, ska_byts, pkb_byts, skb_byts, pa_byts, pb_byts, ssa_byts, ssb_byts, ma_byts, mb_byts;
	int rtn;
	// init drng_seed using nonce
	nonce = (unsigned char *)malloc(SEED_LENGTH_BYTE);
	for (int i = 0; i < SEED_LENGTH_BYTE / 4; i++)
	{
		memcpy(nonce + 4 * i, "seed", 4);
	}
	init_random_number(&drng_seed, nonce, SEED_LENGTH_BYTE);
	// open KAT_KEX_AlgorithmInstance.txt
	sprintf(algoname_output, "KAT_KEX_%s.txt", ALGORITHM_INSTANCE);
	if ((file_output = fopen(algoname_output, "wb")) == NULL)
		return KAT_KEX_FOPEN_ERROR;

	round = kex_get_rounds();
	pka_byts = kex_get_pk_byts();
	ska_byts = kex_get_sk_byts();
	pkb_byts = kex_get_pk_byts();
	skb_byts = kex_get_sk_byts();
	pa_byts = kex_get_pa_byts();
	pb_byts = kex_get_pb_byts();
	ssa_byts = kex_get_ss_byts();
	ssb_byts = kex_get_ss_byts();
	ma_byts = kex_get_bw_byts();
	mb_byts = kex_get_bw_byts();
	pka = (unsigned char *)calloc(pka_byts, sizeof(unsigned char));
	ska = (unsigned char *)calloc(ska_byts, sizeof(unsigned char));
	pkb = (unsigned char *)calloc(pkb_byts, sizeof(unsigned char));
	skb = (unsigned char *)calloc(skb_byts, sizeof(unsigned char));
	pa = (unsigned char *)calloc(pa_byts, sizeof(unsigned char));
	pb = (unsigned char *)calloc(pb_byts, sizeof(unsigned char));
	ssa = (unsigned char *)calloc(ssa_byts, sizeof(unsigned char));
	ssb = (unsigned char *)calloc(ssb_byts, sizeof(unsigned char));
	ma = (unsigned char *)calloc(ma_byts, sizeof(unsigned char));
	mb = (unsigned char *)calloc(mb_byts, sizeof(unsigned char));
	seed = (unsigned char *)calloc(SEED_LENGTH_BYTE, sizeof(unsigned char));
	for (int i = 0; i < 10; i++)
	{
		fprintf(file_output, "count = %d\n", i);
		// generate seed using drng_seed
		get_random_number(&drng_seed, seed, SEED_LENGTH_BYTE * 8);
		// print seed
		fprintf(file_output, "seedlen = %08X\n", SEED_LENGTH_BYTE);
		fprintf(file_output, "seed = ");
		fprintstr(file_output, seed, SEED_LENGTH_BYTE);
		fprintf(file_output, "\n");

		/****************the KEX protocol starts here****************/
		// init drng_algorithm using seed
		init_random_number(&drng_algorithm, seed, SEED_LENGTH_BYTE);

		/**************** Initialize by the initiator****************/
		rtn = kex_init_a(pka, &pka_byts, ska, &ska_byts, pa, &pa_byts);
		if (rtn)
		{
			printf("kex_init_a returned %d\n", rtn);
			return KAT_KEX_CRYPTO_FAILURE;
		}
		// print initiator long-term public key
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "pkalen = \n");
		else
			fprintf(file_output, "pkalen = %08llX\n", pka_byts);
		fprintf(file_output, "pka = ");
		fprintstr(file_output, pka, pka_byts);
		fprintf(file_output, "\n");
		// print initiator long-term private key
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "skalen = \n");
		else
			fprintf(file_output, "skalen = %08llX\n", ska_byts);
		fprintf(file_output, "ska = ");
		fprintstr(file_output, ska, ska_byts);
		fprintf(file_output, "\n");
		// print initiator cached data
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "initpalen = \n");
		else
			fprintf(file_output, "initpalen = %08llX\n", pa_byts);
		fprintf(file_output, "initpa = ");
		fprintstr(file_output, pa, pa_byts);
		fprintf(file_output, "\n");

		/**************** Initialize by the responder****************/
		rtn = kex_init_b(pkb, &pkb_byts, skb, &skb_byts, pb, &pb_byts);
		if (rtn)
		{
			printf("kex_init_a returned %d\n", rtn);
			return KAT_KEX_CRYPTO_FAILURE;
		}
		// print responder long-term public key
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "pkblen = \n");
		else
			fprintf(file_output, "pkblen = %08llX\n", pkb_byts);
		fprintf(file_output, "pkb = ");
		fprintstr(file_output, pkb, pkb_byts);
		fprintf(file_output, "\n");
		// print responder long-term private key
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "skblen = \n");
		else
			fprintf(file_output, "skblen = %08llX\n", skb_byts);
		fprintf(file_output, "skb = ");
		fprintstr(file_output, skb, skb_byts);
		fprintf(file_output, "\n");
		// print responder cached data
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "initpblen = \n");
		else
			fprintf(file_output, "initpblen = %08llX\n", pb_byts);
		fprintf(file_output, "initpb = ");
		fprintstr(file_output, pb, pb_byts);
		fprintf(file_output, "\n");

		/****************   Send by the initiator  ****************/
		rtn = kex_send_a(pkb, pkb_byts, pa, &pa_byts, ma, &ma_byts);
		if (rtn)
		{
			printf("kex_send_a returned %d\n", rtn);
			return KAT_KEX_CRYPTO_FAILURE;
		}
		// print initiator cached data
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "sendpalen = \n");
		else
			fprintf(file_output, "sendpalen = %08llX\n", pa_byts);
		fprintf(file_output, "sendpa = ");
		fprintstr(file_output, pa, pa_byts);
		fprintf(file_output, "\n");
		// print message sent by initiator
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "sendmalen = \n");
		else
			fprintf(file_output, "sendmalen = %08llX\n", ma_byts);
		fprintf(file_output, "sendma = ");
		fprintstr(file_output, ma, ma_byts);
		fprintf(file_output, "\n");

		/****************   Send by the responder  ****************/
		rtn = kex_send_b(pka, pka_byts, ma, ma_byts, pb, &pb_byts, mb, &mb_byts, ssb, &ssb_byts);
		if (rtn)
		{
			printf("kex_send_b returned %d\n", rtn);
			return KAT_KEX_CRYPTO_FAILURE;
		}
		// print responder cached data
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "sendpblen = \n");
		else
			fprintf(file_output, "sendpblen = %08llX\n", pb_byts);
		fprintf(file_output, "sendpb = ");
		fprintstr(file_output, pb, pb_byts);
		fprintf(file_output, "\n");
		// print message sent by responder
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "sendmblen = \n");
		else
			fprintf(file_output, "sendmblen = %08llX\n", mb_byts);
		fprintf(file_output, "sendmb = ");
		fprintstr(file_output, mb, mb_byts);
		fprintf(file_output, "\n");

		/**************** Respond by the initiator ****************/
		rtn = kex_resp_a(mb, mb_byts, pa, &pa_byts, ma, &ma_byts, ssa, &ssa_byts);
		if (rtn)
		{
			printf("kex_resp_a returned %d\n", rtn);
			return KAT_KEX_CRYPTO_FAILURE;
		}
		// print message sent by initiator
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "respmalen = \n");
		else
			fprintf(file_output, "respmalen = %08llX\n", ma_byts);
		fprintf(file_output, "respma = ");
		fprintstr(file_output, ma, ma_byts);
		fprintf(file_output, "\n");

		if (round == 3)
		{
			/**************** Respond by the responder ****************/
			rtn = kex_resp_b(ma, ma_byts, pa, &pb_byts, mb, &mb_byts, ssb, &ssb_byts);
			if (rtn)
			{
				printf("kex_resp_b returned %d\n", rtn);
				return KAT_KEX_CRYPTO_FAILURE;
			}
		}
		// print shared secret key
		if (OUTPUT_BLANK_TEST_VECTORS)
			fprintf(file_output, "sslen = \n");
		else
			fprintf(file_output, "sslen = %08llX\n", ssa_byts);
		fprintf(file_output, "ss = ");
		fprintstr(file_output, ssa, ssa_byts);
		fprintf(file_output, "\n");
		// verify two shared secret keys
		if (memcmp(ssa, ssb, ssa_byts))
		{
			printf("initiator shared secret key != responder shared secret key\n");
			return KAT_KEX_CRYPTO_FAILURE;
		}
		fprintf(file_output, "\n");
	}
	free(seed);
	free(mb);
	free(ma);
	free(ssb);
	free(ssa);
	free(pb);
	free(pa);
	free(skb);
	free(pkb);
	free(ska);
	free(pka);
	free(nonce);
	// close KAT_KEX_AlgorithmInstance.txt
	fclose(file_output);
	return KAT_KEX_SUCCESS;
}

void fprintstr(FILE *file_output, unsigned char *msg, unsigned long long len)
{
	for (unsigned long long i = 0; i < len; i++)
		fprintf(file_output, "%02X", msg[i]);
}