/*
The software is provided by the Institute of Commercial Cryptography Standards 
(ICCS), and is used for algorithm submissions in the Next-generation Commercial 
Cryptographic Algorithms Program. 

ICCS doesn't represent or warrant that the operation of the software will be 
uninterrupted or error-free in all cases. ICCS will take no responsibility for 
the use of the software or the results thereof, if the software is used for any 
other purposes.
*/

#ifndef AUXFUNC_H
#define AUXFUNC_H

#ifdef __cplusplus
extern "C"
{
#endif

    /// @brief The SM3 cryptographic hash function
    /// @param digestbitlen Total bits (only 256) of output digest
    /// @param message Base address of input message byte array
    /// @param messagebitlen Total bits of input message
    /// @param digest Base address of output message digest byte array
    /// @return 0 for success, others for error
    int sm3hash(int digestbitlen,const unsigned char *message, unsigned long long messagebitlen, unsigned char *digest);
    
    /// @brief A pseudo-hash function constructed using SM3 and HMAC to generate a 512/768/1024-bit hash value
    ///        It is only used for correctness verification of implementations, and does not guarantee security   
    /// @param digestbitlen Total bits (only 512/768/1024) of output digest
    /// @param message Base address of input message byte array
    /// @param messagebitlen Total bits of input message
    /// @param digest Base address of output message digest byte array
    /// @return 0 for success, others for error
    int pseudohash(int digestbitlen, const unsigned char *message, unsigned long long messagebitlen, unsigned char *digest);

    /// @brief An eXtensible Output Function (XOF) to generate message digests of any bit length no less than 1024
    ///        It is only used for correctness verification of implementations, and does not guarantee security
    /// @param digestbitlen Total bits of output digest
    /// @param message Base address of input message byte array
    /// @param messagebitlen Total bits of input message
    /// @param digest Base address of output message digest byte array
    /// @return 0 for success, others for error
    int pseudoXOF(int digestbitlen, const unsigned char *message, unsigned long long messagebitlen, unsigned char *digest);
    
#ifdef __cplusplus
}
#endif

#endif